from distutils.core import setup

setup(
    name='pg4nosql',
    version='0.2.4',
    packages=['pg4nosql'],
    url='https://github.com/cansik/pg4nosql',
    license='MIT License',
    author='Florian Bruggisser',
    author_email='florian@nexpose.ch',
    description='A simple psycopg2 based wrapper for nosql like database interaction with python.',
    download_url='https://github.com/cansik/pg4nosql/tarball/0.2.4',
    keywords=['postgres', 'nosql', 'wrapper', 'database', 'json'],
    classifiers=[],
)
